/*-------------------------------------------------------------------------
 *
 * pg_policy_d.h
 *    Macro definitions for pg_policy
 *
 * Portions Copyright (c) 1996-2018, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_POLICY_D_H
#define PG_POLICY_D_H

#define PolicyRelationId 3256

#define Anum_pg_policy_polname 1
#define Anum_pg_policy_polrelid 2
#define Anum_pg_policy_polcmd 3
#define Anum_pg_policy_polpermissive 4
#define Anum_pg_policy_polroles 5
#define Anum_pg_policy_polqual 6
#define Anum_pg_policy_polwithcheck 7

#define Natts_pg_policy 7


#endif							/* PG_POLICY_D_H */
